@@ -1,107 +1,18 @@
-from .block import Block, reduce_blocks
-from .face import Face, create_face_from_diagonals, split_face
+from .block import Block
+from .blockfunctions import reduce_blocks
+from .face import Face
+from .facefunctions import create_face_from_diagonals, split_face, get_outer_faces
 import math 
 from itertools import product, combinations
 from tqdm import trange
 import numpy as np 
 import pandas as pd
-from typing import List, NamedTuple
+from typing import List
 import math
 from .point_match import point_match
 from copy import deepcopy
 
 
-def get_outer_faces(block1:Block):
-    """Get the outer faces of a block
-
-    Args:
-        block1 (Block): A plot3D block
-
-    Returns:
-        List[Face]: Non matching faces of the block 
-        List[(Face,Face)]: Matching faces inside the block 
-    """
-    I = [0,block1.IMAX-1]               # Python index starts at 0, need to subtract 1 for it to get the i,j,k
-    J = [0,block1.JMAX-1]
-    K = [0,block1.KMAX-1]
-    # Create the outer faces        
-    faces = list()
-    face = Face(4)
-    i=I[0]
-    for j in J:
-        for k in K:
-            face.add_vertex(block1.X[i,j,k], block1.Y[i,j,k], block1.Z[i,j,k],i,j,k)
-    
-    faces.append(face)
-    face = Face(4)
-    i=I[1]
-    for j in J:
-        for k in K:
-            face.add_vertex(block1.X[i,j,k], block1.Y[i,j,k], block1.Z[i,j,k],i,j,k)
-    
-    faces.append(face)
-    face = Face(4)
-    j=J[0]
-    for i in I:
-        for k in K:
-            face.add_vertex(block1.X[i,j,k], block1.Y[i,j,k], block1.Z[i,j,k],i,j,k)
-    
-    faces.append(face)
-    face = Face(4)
-    j=J[1]
-    for i in I:
-        for k in K:
-            face.add_vertex(block1.X[i,j,k], block1.Y[i,j,k], block1.Z[i,j,k],i,j,k)
-
-    faces.append(face)
-    face = Face(4)
-    k=K[0]
-    for i in I:
-        for j in J:
-            face.add_vertex(block1.X[i,j,k], block1.Y[i,j,k], block1.Z[i,j,k],i,j,k)
-    
-    faces.append(face)
-    face = Face(4)
-    k=K[1]
-    for i in I:
-        for j in J:
-            face.add_vertex(block1.X[i,j,k], block1.Y[i,j,k], block1.Z[i,j,k],i,j,k)
-    faces.append(face)
-
-    # Check if faces match each other
-    matching = list()
-    non_matching = list()
-    for i in range(len(faces)):
-        matchFound = False
-        for j in range(len(faces)):
-            if (i!=j and faces[i].vertices_equals(faces[j])):
-                matching.append((i,j))
-                matchFound = True
-        if not matchFound:
-            non_matching.append(faces[i]) # these are guaranteed to be exterior 
-    matching = list(unique_pairs(matching))
-    matching = [(faces[i],faces[j]) for i,j in matching]
-    
-    # Make sure normals do not intersect 
-    # block_center_to_face_center =  block1.cx
-    return non_matching, matching # these should be the outer faces
-
-def unique_pairs(listOfItems:list):
-    """Checks if an item is not already in the list 
-    
-    Args:
-        listOfItems (list): list of combinations e.g. (1,2),(3,4),(2,1)
-
-    Yields:
-        unique pair: [description]
-    """
-    seen = set()  #use set to keep track of already seen items, sets provide O(1) lookup  
-    for x,y in listOfItems:
-        if x!=y and (y,x) not in seen:
-            seen.add((x,y)) 
-            yield x,y
-
-
 def find_matching_blocks(block1:Block,block2:Block,block1_outer:List[Face], block2_outer:List[Face],tol:float=1E-6):  
     """Takes two blocks and finds all matching pairs
 
@@ -155,10 +66,6 @@ def find_matching_blocks(block1:Block,block2:Block,block1_outer:List[Face], bloc
             block2_split_faces.clear()
 
     return block_match_indices, block1_outer, block2_outer # Remove duplicates using set and list 
-                    
-        
-
-
 
 def select_multi_dimensional(T:np.ndarray,dim1:tuple,dim2:tuple, dim3:tuple):
     """Takes a block (T) and selects X,Y,Z from the block given a face's dimensions
@@ -183,7 +90,6 @@ def select_multi_dimensional(T:np.ndarray,dim1:tuple,dim2:tuple, dim3:tuple):
     
     return T[dim1[0]:dim1[1], dim2[0]:dim2[1], dim3[0]:dim3[1]]
 
-
 def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:float=1E-6):
     """Get the index of the intersection between two faces located on two different blocks 
         Face1 needs to be the smaller face. 
@@ -335,8 +241,6 @@ def get_face_intersection(face1:Face,face2:Face,block1:Block,block2:Block,tol:fl
         df = pd.DataFrame() # set df to empty dataframe
     return df, split_faces1, split_faces2
 
-
-
 def __filter_block_increasing(df:pd.DataFrame,key1:str):
     """Filters dataframe results of get_face_intersection to make sure both key1 is increasing. 
         When searching through a plot3D we check based on the planes e.g. const i, j, or k 
@@ -384,16 +288,17 @@ def __check_edge(df:pd.DataFrame):
     Returns:
         boolean: True = It is an edge, False = not edge 
     """
-    
-    block = [['i1','j1','k1'],['i2','j2','k2']]
-    # Single edge matching check
-    c=0
-    for b in block:        
-        for k in b: # loop for i,j,k
-            unq = df[k].unique()
-            c += int(len(unq)>1 and len(unq)>2) # example i1 = 1,2,3,4,5
-
-    return c<4 #  If "c" is less than 4 then it's an edge 
+    diagonal = [(df['i1'].min(),df['j1'].min(),df['k1'].min()), (df['i1'].max(),df['j2'].max(),df['k2'].max())]
+    edge1 = diagonal[0]
+    edge2 = diagonal[1]
+    edge_matches = 0 
+    for i in range(3):
+        if edge1[i]==edge2[i]:
+            edge_matches+=1
+    if edge_matches<2:
+        return False
+    else:
+        return True
     
 def combinations_of_nearest_blocks(blocks:List[Block],nearest_nblocks:int=4):
     """Returns the indices of the nearest 6 blocks based on their centroid
@@ -433,23 +338,6 @@ def combinations_of_nearest_blocks(blocks:List[Block],nearest_nblocks:int=4):
                 new_combos.append((i,j))
     return new_combos
     
-
-def find_block_index_in_outer_faces(outer_faces:List[Face], block_indx:int):
-    """Looks through an array of faces (outer_faces). Outer faces can include multiple blocks
-        This function finds the face of corresponding block index
-
-    Args:
-        outer_faces (List[Face]): List of all outer faces for all blocks
-        block_indx (int): block index
-
-    Returns:
-        int: block index within outer_faces array that corresponds to all outer faces of that single block
-    """
-    for i in range(len(outer_faces)):
-        if outer_faces[i]['block'] == block_indx:
-            return i
-    return -1 
-
 def connectivity_fast(blocks:List[Block]):
     """Reduces the size of the blocks by a factor of the minimum gcd. This speeds up finding the connectivity 
 
@@ -599,7 +487,6 @@ def connectivity(blocks:List[Block]):
 
     return face_matches, outer_faces_formatted  
 
-
 def face_matches_to_dict(face1:Face, face2:Face,block1:Block,block2:Block):
     """Makes sure the diagonal of face 1 match the diagonal of face 2
 
@@ -679,4 +566,4 @@ def face_matches_to_dict(face1:Face, face2:Face,block1:Block,block2:Block):
     match['block2']['IMAX'] = int(df.iloc[0]['I'])
     match['block2']['JMAX'] = int(df.iloc[0]['J'])
     match['block2']['KMAX'] = int(df.iloc[0]['K'])
-    return match
\ No newline at end of file
+    return match
